@@ -79,6 +79,7 @@ impl Default for Layer {
 pub enum ChannelMode {
 	Stereo = 0,
 	JointStereo = 1,
+	/// Two independent mono channels
 	DualChannel = 2,
 	SingleChannel = 3,
 }
@@ -89,21 +90,43 @@ impl Default for ChannelMode {
 	}
 }
 
+#[derive(Copy, Clone, PartialEq, Debug)]
+#[allow(missing_docs, non_camel_case_types)]
+/// A rarely-used decoder hint that the file must be de-emphasized
+pub enum Emphasis {
+	None,
+	/// 50/15 ms
+	MS5015,
+	Reserved,
+	/// CCIT J.17
+	CCIT_J17,
+}
+
+impl Default for Emphasis {
+	fn default() -> Self {
+		Self::None
+	}
+}
+
 #[derive(Copy, Clone)]
 pub(crate) struct Header {
-	pub sample_rate: u32,
-	pub channels: u8,
-	pub len: u32,
-	pub data_start: u32,
-	pub samples: u16,
-	pub bitrate: u32,
-	pub version: MpegVersion,
-	pub layer: Layer,
-	pub channel_mode: ChannelMode,
+	pub(crate) sample_rate: u32,
+	pub(crate) channels: u8,
+	pub(crate) len: u32,
+	pub(crate) data_start: u32,
+	pub(crate) samples: u16,
+	pub(crate) bitrate: u32,
+	pub(crate) version: MpegVersion,
+	pub(crate) layer: Layer,
+	pub(crate) channel_mode: ChannelMode,
+	pub(crate) mode_extension: Option<u8>,
+	pub(crate) copyright: bool,
+	pub(crate) original: bool,
+	pub(crate) emphasis: Emphasis,
 }
 
 impl Header {
-	pub fn read(header: u32) -> Result<Self> {
+	pub(crate) fn read(header: u32) -> Result<Self> {
 		let version = match (header >> 19) & 0b11 {
 			0 => MpegVersion::V2_5,
 			2 => MpegVersion::V2,
@@ -147,7 +170,7 @@ impl Header {
 			_ => sample_rate = SAMPLE_RATES[version as usize][sample_rate as usize],
 		}
 
-		let has_padding = ((header >> 9) & 1) != 0;
+		let has_padding = ((header >> 9) & 1) == 1;
 		let mut padding = 0;
 
 		if has_padding {
@@ -162,6 +185,23 @@ impl Header {
 			_ => unreachable!(),
 		};
 
+		let mut mode_extension = None;
+
+		if let ChannelMode::JointStereo = channel_mode {
+			mode_extension = Some(((header >> 4) & 3) as u8);
+		}
+
+		let copyright = ((header >> 3) & 1) == 1;
+		let original = ((header >> 2) & 1) == 1;
+
+		let emphasis = match header & 3 {
+			0 => Emphasis::None,
+			1 => Emphasis::MS5015,
+			2 => Emphasis::Reserved,
+			3 => Emphasis::CCIT_J17,
+			_ => unreachable!(),
+		};
+
 		let data_start = SIDE_INFORMATION_SIZES[version_index][channel_mode as usize] + 4;
 		let samples = SAMPLES[layer_index][version_index];
 
@@ -190,17 +230,21 @@ impl Header {
 			version,
 			layer,
 			channel_mode,
+			mode_extension,
+			copyright,
+			original,
+			emphasis,
 		})
 	}
 }
 
 pub(crate) struct XingHeader {
-	pub frames: u32,
-	pub size: u32,
+	pub(crate) frames: u32,
+	pub(crate) size: u32,
 }
 
 impl XingHeader {
-	pub fn read(reader: &mut &[u8]) -> Result<Self> {
+	pub(crate) fn read(reader: &mut &[u8]) -> Result<Option<Self>> {
 		let reader_len = reader.len();
 
 		let mut header = [0; 4];
@@ -230,7 +274,7 @@ impl XingHeader {
 				let frames = reader.read_u32::<BigEndian>()?;
 				let size = reader.read_u32::<BigEndian>()?;
 
-				Ok(Self { frames, size })
+				Ok(Some(Self { frames, size }))
 			},
 			b"VBRI" => {
 				if reader_len < 32 {
@@ -250,13 +294,9 @@ impl XingHeader {
 				let size = reader.read_u32::<BigEndian>()?;
 				let frames = reader.read_u32::<BigEndian>()?;
 
-				Ok(Self { frames, size })
+				Ok(Some(Self { frames, size }))
 			},
-			_ => Err(FileDecodingError::new(
-				FileType::MP3,
-				"No Xing, LAME, or VBRI header located",
-			)
-			.into()),
+			_ => Ok(None),
 		}
 	}
 }
@@ -267,10 +307,7 @@ mod tests {
 	fn search_for_frame_sync() {
 		fn test(data: &[u8], expected_result: Option<u64>) {
 			use super::search_for_frame_sync;
-			assert_eq!(
-				search_for_frame_sync(&mut Box::new(data)).unwrap(),
-				expected_result
-			);
+			assert_eq!(search_for_frame_sync(&mut &*data).unwrap(), expected_result);
 		}
 
 		test(&[0xFF, 0xFB, 0x00], Some(0));
